{% load url from compat %}
{% if note %}
    <h2>{{ note }}</note>
{% else %}
    <h2>Terms of Service as of {{ tos.created|date:"SHORT_DATE_FORMAT" }}</h2>
{% endif %}

{{ tos.content|safe }}

<h2>Accept Terms of Service?</h2>

<form method="post" action="{% url "tos_check_tos" %}">
    {% csrf_token %}
    {% if next %}
        <input type="hidden" name="next" value="{{next}}">
    {% endif %}
    <input type="submit" name="accept" value="accept">
    <input type="submit" name="accept" value="reject">
</form>
